/******************************************************************************
    CallWave SMS Widget

    Send free SMS text messages worldwide.

    Copyright (C) 2006 CallWave, Inc.  All Rights Reserved.

******************************************************************************/

function SMS_SMSView( onLoginRequired )
{
    try
    {
        this.m_smsBodyOH = 0;
        this.m_showCarriers = true;

        this.m_messageDimensions = new this.MessageDimensions( this, 0 );

        this.m_recipientPhoneNumber = '';

        this.m_confirmationScreen = null;
        this.m_confirmationScreenTimeout = null;
        this.m_confirmationScreenDismissalAction = null;

        this.m_animationBusy = false;

        this.m_timeoutServer = new TimeoutServer( );

        this.m_okButtonDisplayDelayMS = 0;

        var mode = VM_GetPreference( PREF_KEY_SMS_VIEW_MODE );

        var modeId = '';

        if ( mode == this.MODE_INTERNATIONAL )
        {
            document.getElementById( 'internationalRadio' ).checked = true;
            document.getElementById( 'domesticRadio' ).checked = false;

            this.UpdateInternationalCounts( );

            modeId = 'internationalRadio';
        }
        else
        {
            document.getElementById( 'domesticRadio' ).checked = true;
            document.getElementById( 'internationalRadio' ).checked = false;

            this.UpdateDomesticCounts( );

            modeId = 'domesticRadio';
        }

        var lastContact = VM_GetPreference( PREF_KEY_LAST_CONTACT_KEY );
        if ( lastContact && lastContact.length > 0 )
        {
            var contactPopup = document.getElementById( "contactPopup" );
            SMS_UTILS_SelectItemInCustomPopup( lastContact, contactPopup, document.getElementById( "contactMenuText" ) );

            this.OnContactChanged( );
        }

        this.OnModeChanged( modeId );
        this.m_onLoginRequired = onLoginRequired;
        // set up some closures
        var self = this;
        var onSendSMSSucceeded = function( responseMessage )
        {
            self.OnSendSMSSucceeded( responseMessage );
        };

        var onSendSMSFailed = function( statusMessage )
        {
            self.OnSendSMSFailed( statusMessage );
        };

        this.m_smsSendRequestor = new AH_SMSSendRequestor( VM_WIDGET_SOURCE,
                                                           VM_WIDGET_VERSION,
                                                           VM_WIDGET_OS_VERSION,
                                                           onSendSMSSucceeded,
                                                           onSendSMSFailed );

        var handleEventPop = function( responseMessage )
        {
            if ( responseMessage.m_smsSendPop !== null )
            {
                self.HandleEventPop( responseMessage.m_smsSendPop );
            }
        };

        this.m_smsSendPopRequestor = new AH_SMSSendPopRequestor( VM_WIDGET_SOURCE,
                                                                 VM_WIDGET_VERSION,
                                                                 VM_WIDGET_OS_VERSION,
                                                                 handleEventPop );

        this.m_crmContentRequestor = null;
    }
    catch ( e )
    {
        EX_ASSERT_NO_EXCEPTIONS( e, "SMS_SMSView::SMS_SMSView( )" );
    }
}

SMS_SMSView.prototype =
{
    /******************************************************************************
      Constants
    ******************************************************************************/
    MODE_DOMESTIC                       : "free",
    MODE_INTERNATIONAL                  : "worldwide",
    ERROR_MSG_COLOR                     : '#bf0000',
    GENERIC_FAILURE_MESSAGE             : "Message not sent. Please confirm your Internet connection.",

    CONFIRMATION_SCREEN_TIMEOUT_MS      : 4 * 60 * 60 * 1000,

    SMS_SCREEN_HEIGHT : 130,
    CONFIRMATION_SCREEN_MARGIN : 10,

    MAXIMUM_SCREEN_HEIGHT : 400,

    MessageDimensions : function( parent, smsBodyOH, maxMessageParts )
    {
        // private constants
        var MAX_MESSAGE_PARTS  = ( maxMessageParts ? maxMessageParts : 5 );
        var MAX_MESSAGE_LENGTH = 158;
        var CARRIER_OVERHEAD   = 2;

        this.m_parent = parent;

        this.MAX_INTERNATIONAL_MESSAGE_LENGTH        = 160;
        this.MAX_UNICODE_MESSAGE_LENGTH              = 70;

        var senderMobile = SMS_MobileNumberControlPersistentState.prototype.CreateFromString( VM_GetPreference( PREF_KEY_SENDER_MOBILE ) ).m_numberFieldData;
        var senderEmail  = VM_GetPreference( PREF_KEY_SENDER_EMAIL );

        var replyAddressLength = 38;
        if ( senderMobile && ( senderMobile.length > 0 ) )
        {
            replyAddressLength = senderMobile.length + 23;
        }
        else if ( senderEmail && ( senderEmail.length > 0 ) )
        {
            replyAddressLength = senderEmail.length;
        }
        var singlePartOverheadLength     = replyAddressLength + CARRIER_OVERHEAD + smsBodyOH;
        this.maxSinglePartLength         = MAX_MESSAGE_LENGTH - singlePartOverheadLength;

        var multiPartOverheadLength      = singlePartOverheadLength + "x of x: ".length;
        this.maxMultiPartLength          = MAX_MESSAGE_LENGTH - multiPartOverheadLength;
        this.maxTotalMessageLength       = MAX_MESSAGE_PARTS * this.maxMultiPartLength;

        this.MAX_INTERNATIONAL_MESSAGE_PART_LENGTH   = this.MAX_INTERNATIONAL_MESSAGE_LENGTH - 8;
        this.MAX_UNICODE_MESSAGE_PART_LENGTH         = this.MAX_UNICODE_MESSAGE_LENGTH - 8;
        this.MAX_TOTAL_INTERNATIONAL_MESSAGE_LENGTH  = this.MAX_MESSAGE_PARTS * this.MAX_INTERNATIONAL_MESSAGE_PART_LENGTH;
        this.MAX_TOTAL_UNICODE_MESSAGE_LENGTH        = this.MAX_MESSAGE_PARTS * this.MAX_UNICODE_MESSAGE_PART_LENGTH;

        this.LimitDomesticMessageLength = function ( messageObj )
        {
            var message = messageObj.value;

            var messageLength = message.length;

            if ( messageLength <= this.maxTotalMessageLength )
            {
                return messageLength;
            }

            messageObj.value = message.substr( 0, this.maxTotalMessageLength );

            return this.maxTotalMessageLength;
        }

        this.DetermineDomesticMessagePartCount = function ( messageLength )
        {
            if ( messageLength > this.maxSinglePartLength )
            {
                return Math.ceil( messageLength / this.maxMultiPartLength );
            }

            return 1;
        }

        this.LimitInternationalMessageLength = function ( messageObj )
        {
            if ( messageObj.value.length > this.MAX_TOTAL_INTERNATIONAL_MESSAGE_LENGTH )
            {
                messageObj.value = messageObj.value.substr( 0, this.MAX_TOTAL_INTERNATIONAL_MESSAGE_LENGTH );
            }
        };

        this.DetermineIntlMessageUnicodePartCount = function( messageLength )
        {
            if ( messageLength > this.MAX_TOTAL_UNICODE_MESSAGE_LENGTH )
            {
                this.SetErrorMessage( 'Unicode messages can only have ' + this.MAX_TOTAL_UNICODE_MESSAGE_LENGTH + ' characters' );
            }

            if ( messageLength > this.MAX_UNICODE_MESSAGE_LENGTH )
            {
                return Math.ceil( messageLength / this.MAX_UNICODE_MESSAGE_PART_LENGTH );
            }

            return 1;
        };

        this.DetermineIntlMessagePartCount = function( messageLength )
        {
            if ( messageLength > this.MAX_INTERNATIONAL_MESSAGE_LENGTH )
            {
                return Math.ceil( messageLength / this.MAX_INTERNATIONAL_MESSAGE_PART_LENGTH );
            }

            return 1;
        };
    },

    SetRecipientPhoneNumber : function( recipientPhoneNumber )
    {
        document.getElementById( 'recipientPhoneNumber' ).value = recipientPhoneNumber;
        this.m_recipientPhoneNumber = recipientPhoneNumber;
    },

    /**************************************************************************
      View repainting methods
     **************************************************************************/

    reset:function( )
    {
        try
        {
            document.getElementById( 'WIDGET_smsViewFrame' ).style.display = 'none';
            document.getElementById( 'messageText' ).value = '';
            this.SetRecipientPhoneNumber( '' );

            SMS_UTILS_SelectFirstItemInCustomPopup( document.getElementById( "contactPopup" ), document.getElementById( "contactMenuText" ) );

            if ( this.m_showCarriers )
            {
                SMS_UTILS_SelectFirstItemInCustomPopup( document.getElementById( "carrierPopup" ), document.getElementById( "carrierMenuText" ) );
            }

            this.OnModeChanged( 'domesticRadio' );

            document.getElementById( 'domesticRadio' ).checked = true;
            document.getElementById( 'internationalRadio' ).checked = false;

            this.UpdateInternationalCounts( );
            this.UpdateDomesticCounts( );
        }
        catch ( e )
        {
            EX_Log( "SMS_SMSView::reset()" + e.message );
        }
    },

    refresh:function( )
    {
        document.getElementById( 'WIDGET_smsViewFrame' ).style.display = 'block';

        document.getElementById( 'messageText' ).focus( );
        document.getElementById( 'messageText' ).select( );
    },

    UpdateCounts : function( )
    {
        if ( document.getElementById( 'internationalRadio' ).checked )
        {
            this.UpdateInternationalCounts( );
        }
        else
        {
            this.UpdateDomesticCounts( );
        }
    },

    /******************************************************************************
      OnModeChanged

        Keaka note: there is a bug in 10.4.3 where the other radio buttons aren't
                    unchecked when a new one is checked.  This was fixed in 10.4.4, but
                    we have to manually uncheck the other button here.

    ******************************************************************************/
    OnModeChanged : function( elementId )
    {
        try
        {
            if ( elementId == "internationalRadio" )
            {
                document.getElementById( 'domesticRadio' ).checked = false;
                document.getElementById( 'carrierDiv' ).style.display = 'none';
                VM_SetPreference( PREF_KEY_SMS_VIEW_MODE, this.MODE_INTERNATIONAL );

                this.UpdateInternationalCounts( );
            }
            else
            {
                document.getElementById( 'internationalRadio' ).checked = false;

                if ( this.m_showCarriers )
                {
                    document.getElementById( 'carrierDiv' ).style.display = 'block';
                }
                else
                {
                    document.getElementById( 'carrierDiv' ).style.display = 'none';
                }

                VM_SetPreference( PREF_KEY_SMS_VIEW_MODE, this.MODE_DOMESTIC );

                this.UpdateDomesticCounts( );
            }
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_SMSView::OnModeChanged( )" );
        }
    },

    OnLoginSucceeded : function( smsBodyOH, maxSMSParts, showCarriers, okButtonDisplayDelayMS )
    {
        try
        {
            this.m_messageDimensions = new this.MessageDimensions( this, smsBodyOH, maxSMSParts );

            this.m_showCarriers = showCarriers;
            this.m_okButtonDisplayDelayMS = okButtonDisplayDelayMS;
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'SMS_SMSView::OnLoginSucceeded( )' );
        }
    },

    /******************************************************************************
      OnRecipientCarrierChanged

        Handler for recipient carrier selection-change events

    ******************************************************************************/
    OnRecipientCarrierChanged : function( elem )
    {
        var chosenCarrierId = elem.options[ elem.selectedIndex ].value;

        if ( chosenCarrierId != INVALID_CARRIER_ID )
        {
            var chosenCarrier = elem.options[ elem.selectedIndex ].text;

            document.getElementById( "carrierMenuText" ).innerText = chosenCarrier;
        }
        else
        {
            SMS_UTILS_SelectFirstItemInCustomPopup( elem, document.getElementById( "carrierMenuText" ) );
        }

        document.getElementById( 'messageText' ).focus( );
        document.getElementById( 'messageText' ).select( );
    },

    /******************************************************************************
      OnContactChanged

        Handler for contact change events

    ******************************************************************************/
    OnContactChanged : function( )
    {
        try
        {
            var contactPopup = document.getElementById( "contactPopup" );

            var chosenContact = contactPopup.options[ contactPopup.selectedIndex ].text;

            if ( chosenContact.length <= 17 )
            {
                document.getElementById( "contactMenuText" ).innerText = chosenContact;
            }
            else
            {
                var trimmedContact = chosenContact.substring( 0, 15 ) + "...";
                document.getElementById( "contactMenuText" ).innerText = trimmedContact;
            }

            var chosenContactNumber = contactPopup.options[ contactPopup.selectedIndex ].value;
            if ( chosenContactNumber == CONTACT_NONE_SELECTED_VALUE )
            {
                // We want the number fields to be blank if they select "None" from the contact menu.
                chosenContactNumber = "";
            }
            else
            {
                document.getElementById( 'messageText' ).focus( );
                document.getElementById( 'messageText' ).select( );
            }

            var numberLength = chosenContactNumber.length;

            this.SetRecipientPhoneNumber( chosenContactNumber );

            this.SelectCarrierForNumber( );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_SMSView::OnContactChanged( )" );
        }
    },

    /******************************************************************************
      OnRecipientNumberChanged

        Handler for recipient number change events

    ******************************************************************************/
    OnRecipientNumberChanged : function( )
    {
        try
        {
            var recipientPhoneNumber = document.getElementById( 'recipientPhoneNumber' ).value;

            if ( recipientPhoneNumber == this.m_recipientPhoneNumber )
            {
                return;
            }

            this.m_recipientPhoneNumber = recipientPhoneNumber;

            SMS_UTILS_SelectFirstItemInCustomPopup( document.getElementById( "contactPopup" ),
                                                    document.getElementById( "contactMenuText" ) );

            this.SelectCarrierForNumber( );
        }
        catch ( e )
        {
            EX_Log( "SMS_SMSView::OnRecipientNumberChanged( )\n" + e.message );
        }
    },

    /******************************************************************************
      SynchronizeContactSelectionToNumber

        Sets the contact selection appropriately according to the number.

    ******************************************************************************/
    SynchronizeContactSelectionToNumber : function( )
    {
        try
        {
            var recipientPhoneNumber = document.getElementById( 'recipientPhoneNumber' ).value;

            var contactPopup = document.getElementById( "contactPopup" );

            for ( i = 0; i < contactPopup.options.length; ++i )
            {
                if ( contactPopup.options[ i ].value == recipientPhoneNumber )
                {
                    SMS_UTILS_SelectItemInCustomPopup( contactPopup.options[ i ].value,
                                                       contactPopup,
                                                       document.getElementById( "contactMenuText" ) );

                    return;
                }
            }

            SMS_UTILS_SelectFirstItemInCustomPopup( contactPopup,
                                                    document.getElementById( "contactMenuText" ) );
        }
        catch ( e )
        {
            EX_Log( "SMS_SMSView::SynchronizeContactSelectionToNumber( )\n" + e.message );
        }
    },

    /******************************************************************************
      SelectCarrierForNumber

        Selects the carrier of the recipient phone number

    ******************************************************************************/
    SelectCarrierForNumber : function( )
    {
        try
        {
            if ( ! this.m_showCarriers )
            {
                return;
            }

            var recipientPhoneNumber = document.getElementById( 'recipientPhoneNumber' ).value;

            if ( ! recipientPhoneNumber || recipientPhoneNumber.length == 0 )
            {
                return;
            }

            var savedCarrier = VM_GetPreference( recipientPhoneNumber );

            if ( savedCarrier && savedCarrier.length > 0 )
            {
                var savedCarrierId = INVALID_CARRIER_ID;

                if ( savedCarrier.match( /^\d+$/  ) )
                {
                    savedCarrierId = parseInt( savedCarrier );
                }
                else
                {
                    savedCarrierId = SMS_FindCarrierId( savedCarrier );
                }

                SMS_UTILS_SelectItemInCustomPopup( savedCarrierId, document.getElementById( 'carrierPopup' ), document.getElementById( 'carrierMenuText' ) );
                return;
            }

            SMS_UTILS_SelectFirstItemInCustomPopup( document.getElementById( 'carrierPopup' ),
                                                    document.getElementById( "carrierMenuText" ) );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_SMSView::SelectCarrierForNumber( )" );
        }
    },

    /******************************************************************************
      UpdateInternationalCounts

        Updates the international message counts

    ******************************************************************************/
    UpdateInternationalCounts : function( )
    {
        try
        {
            var message = document.getElementById( 'messageText' );

            this.m_messageDimensions.LimitInternationalMessageLength( message );

            document.getElementById( 'characterCount' ).innerHTML = "Chars: " + message.value.length;
            var partCount;
            if ( SMS_UTILS_StringRequiresUnicodeEncoding( message.value ) )
            {
                document.getElementById( 'unicodeIndicator' ).style.display = 'block';

                partCount = this.m_messageDimensions.DetermineIntlMessageUnicodePartCount( message.value.length );
            }
            else
            {
                document.getElementById( 'unicodeIndicator' ).style.display = 'none';

                partCount = this.m_messageDimensions.DetermineIntlMessagePartCount( message.value.length );
            }

            document.getElementById( 'partCount' ).innerHTML = "Parts: " + partCount;
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_SMSView::UpdateInternationalCounts( )" );
        }
    },

    /******************************************************************************
      UpdateDomesticCounts

        Updates the domestic message counts

    ******************************************************************************/
    UpdateDomesticCounts : function( )
    {
        try
        {
            var message = document.getElementById( 'messageText' );
            var messageLength = this.m_messageDimensions.LimitDomesticMessageLength( message );

            document.getElementById( 'characterCount' ).innerHTML = "Chars: " + messageLength;
            document.getElementById( 'partCount' ).innerHTML = "Parts: " + this.m_messageDimensions.DetermineDomesticMessagePartCount( messageLength );
            document.getElementById( 'unicodeIndicator' ).style.display = 'none';
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_SMSView::UpdateDomesticCounts( )" );
        }
    },


    /******************************************************************************
     ******************************************************************************

      Message Sending functions

     ******************************************************************************
    ******************************************************************************/
    SendSMS:function( sendToPhoneNumber, sendToCarrierID, message )
    {
        try
        {
            this.ShowConfirmationScreen( SMS_ConfirmationScreen.prototype.STATE_SCREEN_IN_PROGRESS_VIEW );

            this.m_smsSendPopRequestor.Start( VM_GetPreference( PREF_KEY_ACCOUNT ),
                                              VM_GetPreference( PREF_KEY_U_STRING ) );

            this.m_smsSendRequestor.Start( VM_GetPreference( PREF_KEY_ACCOUNT ),
                                           VM_GetPreference( PREF_KEY_U_STRING ),
                                           sendToPhoneNumber,
                                           sendToCarrierID,
                                           message );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'SMS_SMSView::SendSMS( )' );
        }
    },

    OnSendSMSSucceeded: function( responseMessage )
    {
        try
        {
            switch ( responseMessage.m_status )
            {
            case MSG_ResponseStatus.SUCCESS:
                {
                    // reset message-specific presentation
                    document.getElementById( 'messageText' ).value = '';
                    this.UpdateInternationalCounts( );
                    this.UpdateDomesticCounts( );

                    // show status from server
                    this.SetSuccessMessage( responseMessage.m_statusMsg );
                }
                break;

            case MSG_ResponseStatus.SSMS_LOGIN_REQUIRED:
                {
                    this.SetErrorMessage( responseMessage.m_statusMsg );

                    var self = this;
                    this.m_confirmationScreenDismissalAction = function( )
                    {
                        self.m_onLoginRequired( );
                    };
                }
                break;

            case MSG_ResponseStatus.GENERAL_ERROR:
            default:
                {
                    var alertMessage = this.GENERIC_FAILURE_MESSAGE;

                    if ( responseMessage.m_statusMsg != null && responseMessage.m_statusMsg.length != 0 )
                    {
                        alertMessage = responseMessage.m_statusMsg;
                    }

                    this.SetErrorMessage( alertMessage );

                    break;
                }
            }
        }
        catch ( e )
        {
            EX_Log( "SMS_SMSView::OnSendSMSSucceeded( ) : " + e.message ? e.message : UTILS.dump(e) );
        }
    },

    OnSendSMSFailed: function( statusMessage )
    {
        try
        {
            this.SetErrorMessage( this.GENERIC_FAILURE_MESSAGE );

            EX_Log( "SMS_SMSView::OnSendSMSFailed( ) : " + statusMessage );
        }
        catch ( e )
        {
            EX_Log( "SMS_SMSView::OnSendSMSFailed( ) : " + e.message );
        }
    },

    HandleEventPop : function( eventPop )
    {
        try
        {
            var self = this;
            var handleCRMContent = function( responseMessage )
            {
                if ( responseMessage.m_content !== null )
                {
                    self.HandleCRMContent( eventPop, responseMessage.m_content );
                }
            };

            this.m_crmContentRequestor = new AH_CRMContentRequestor( VM_WIDGET_SOURCE,
                                                                     VM_WIDGET_VERSION,
                                                                     VM_WIDGET_OS_VERSION,
                                                                     handleCRMContent );

            this.m_crmContentRequestor.Start( VM_GetPreference( PREF_KEY_ACCOUNT ),
                                              VM_GetPreference( PREF_KEY_U_STRING ),
                                              eventPop.m_uriStem,
                                              eventPop.m_webServerName );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_SMSView::HandleEventPop( )" );
        }
    },

    HandleCRMContent : function( eventPop, crmContent )
    {
        try
        {
            var self = this;

            if ( this.m_animationBusy )
            {
                var delayedOperation = function( )
                {
                    self.HandleCRMContent( eventPop, crmContent );
                };

                this.m_timeoutServer.RequestTimeout( delayedOperation, 100 /* delayMS */ );

                return;
            }

            if ( this.m_confirmationScreen === null )
            {
                return;
            }

            this.m_animationBusy = true;

            var completionOperation = function( )
            {
                self.m_confirmationScreen.SetPromotionalContent( eventPop.m_widthPixels, crmContent.m_htmlContent );

                self.m_animationBusy = false;
            };

            var maxSizeWithoutResizing = this.SMS_SCREEN_HEIGHT -
                                         SMS_ConfirmationScreen.prototype.SCREEN_TOP_OFFSET -
                                         2 * this.CONFIRMATION_SCREEN_MARGIN;

            this.m_animationBusy = true;

            if ( eventPop.m_heightPixels > maxSizeWithoutResizing )
            {
                this.GrowBy( Math.min( eventPop.m_heightPixels - maxSizeWithoutResizing, this.MAXIMUM_SCREEN_HEIGHT ), completionOperation );
            }
            else
            {
                completionOperation( );
            }
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_SMSView::HandleCRMContent( )" );
        }
    },

    GrowBy : function( heightDelta, completionOperation )
    {
        try
        {
            var currentHeight = parseInt( document.getElementById( 'WIDGET_smsSendScreen' ).style.height );

            var targetHeight = heightDelta + currentHeight;

            var self = this;

            function SetHeight( height )
            {
                document.getElementById( 'WIDGET_smsSendScreen' ).style.height = height;

                self.m_confirmationScreen.SetHeight( height );
            }

            function AnimationStep( )
            {
                if ( currentHeight + SMS_ConfirmationScreen.prototype.ANIMATION_STEP >= targetHeight )
                {
                    SetHeight( targetHeight );

                    completionOperation( );

                    return;
                }

                currentHeight += SMS_ConfirmationScreen.prototype.ANIMATION_STEP;

                SetHeight( currentHeight );

                setTimeout( AnimationStep, SMS_ConfirmationScreen.prototype.ANIMATION_INTERVAL_MS );
            }

            AnimationStep( );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_SMSView::GrowBy( )" );
        }
    },

    SetErrorMessage : function( message )
    {
        try
        {
            this.ShowConfirmationScreen( SMS_ConfirmationScreen.prototype.STATE_SCREEN_ERROR_VIEW );

            var self = this;
            var completionOperation = function( )
            {
                self.m_confirmationScreen.SetPromotionalContent( 70, '<img src="Images/alert_icon.png" />' );
            };

            this.m_confirmationScreen.SetErrorMessage( message, completionOperation );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_SMSView::SetErrorMessage( )" );
        }
    },

    SetSuccessMessage : function( message )
    {
        try
        {
            this.ShowConfirmationScreen( SMS_ConfirmationScreen.prototype.STATE_SCREEN_SUCCESS_VIEW );

            document.getElementById( 'WIDGET_smsSendSuccessViewButton' ).style.display = 'none';

            var self = this;
            var completionOperation = function( )
            {
                function showOkButton( )
                {
                    if ( self.m_confirmationScreen !== null )
                    {
                        document.getElementById( 'WIDGET_smsSendSuccessViewButton' ).style.display = 'block';
                    }
                };

                if ( self.m_okButtonDisplayDelayMS > 0 )
                {
                    setTimeout( showOkButton, self.m_okButtonDisplayDelayMS );
                }
                else
                {
                    showOkButton( );
                }
            };

            this.m_confirmationScreen.SetSuccessMessage( message, completionOperation );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_SMSView::SetSuccessMessage( )" );
        }
    },

    HideConfirmationScreen : function( completionOperation )
    {
        try
        {
            // clear timeouts
            if ( this.m_confirmationScreenTimeout !== null )
            {
                clearTimeout( this.m_confirmationScreenTimeout );
                this.m_confirmationScreenTimeout = null;
            }

            // clean up confirmation screen
            if ( this.m_confirmationScreen === null )
            {
                completionOperation( );
                return;
            }

            var currentHeight = parseInt( document.getElementById( 'WIDGET_smsSendScreen' ).style.height );

            var self = this;

            function AnimationStep( )
            {
                if ( currentHeight - SMS_ConfirmationScreen.prototype.ANIMATION_STEP <= self.SMS_SCREEN_HEIGHT )
                {
                    self.m_confirmationScreen.MoveUpBy( currentHeight - self.SMS_SCREEN_HEIGHT );

                    document.getElementById( 'WIDGET_smsSendScreen' ).style.height = self.SMS_SCREEN_HEIGHT;

                    var animateUpCompletionOperation = function( )
                    {
                        self.m_confirmationScreen.Destructor( );
                        self.m_confirmationScreen = null;

                        completionOperation( );
                    };

                    self.m_confirmationScreen.AnimateUp( animateUpCompletionOperation );

                    return;
                }

                currentHeight -= SMS_ConfirmationScreen.prototype.ANIMATION_STEP;

                document.getElementById( 'WIDGET_smsSendScreen' ).style.height = currentHeight;

                self.m_confirmationScreen.MoveUpBy( SMS_ConfirmationScreen.prototype.ANIMATION_STEP );

                setTimeout( AnimationStep, SMS_ConfirmationScreen.prototype.ANIMATION_INTERVAL_MS );
            }

            AnimationStep( );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_SMSView::HideConfirmationScreen( )" );
        }
    },

    ShowConfirmationScreen : function( initialState )
    {
        try
        {
            if ( this.m_confirmationScreen !== null )
            {
                // already visible
                return;
            }

            this.m_animationBusy = true;

            var self = this;
            var completionOperation = function( )
            {
                var confirmationScreenTimeoutOperation = function( )
                {
                    self.OnDismissConfirmationScreen( );
                };

                self.m_confirmationScreenTimeout = setTimeout( confirmationScreenTimeoutOperation,
                                                               self.CONFIRMATION_SCREEN_TIMEOUT_MS );

                // disable all the control regions in the SMS screen
                self.UpdateDashboardControlRegions( false );

                self.m_animationBusy = false;
            };

            this.m_confirmationScreen = new SMS_ConfirmationScreen( this.m_timeoutServer, initialState, completionOperation );
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_SMSView::ShowConfirmationScreen( )" );
        }
    },

    UpdateDashboardControlRegions : function( enable )
    {
        try
        {
            var dashboardControls = [ "internationalRadio",
                                      "domesticRadio",
                                      "carrierPopup",
                                      "messageText",
                                      "contactPopup",
                                      "recipientPhoneNumber",
                                      "doButton" ];

            for ( var controlIndex in dashboardControls )
            {
                if ( enable )
                {
                    document.getElementById( dashboardControls[ controlIndex ] ).style[ "-apple-dashboard-region" ] = "dashboard-region(control rectangle);";
                }
                else
                {
                    document.getElementById( dashboardControls[ controlIndex ] ).style[ "-apple-dashboard-region" ] = "none";
                }
            }
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, "SMS_SMSView::UpdateDashboardControlRegions( )" );
        }
    },

    OnDismissConfirmationScreen : function( )
    {
        try
        {
            var self = this;

            if ( this.m_animationBusy )
            {
                var delayedOperation = function( )
                {
                    self.OnDismissConfirmationScreen( );
                };

                this.m_timeoutServer.RequestTimeout( delayedOperation, 100 /* delayMS */ );
                return;
            }

            this.m_animationBusy = true;

            var completionOperation = function( )
            {
                // enable all the control regions in the SMS screen
                self.UpdateDashboardControlRegions( true );

                if ( self.m_confirmationScreenDismissalAction !== null )
                {
                    self.m_confirmationScreenDismissalAction( );
                }

                self.m_confirmationScreenDismissalAction = null;

                self.m_animationBusy = false;
            };

            if ( this.m_crmContentRequestor !== null )
            {
                this.m_crmContentRequestor.Stop( );
                this.m_crmContentRequestor = null;
            }

            this.HideConfirmationScreen( completionOperation );
        }
        catch ( e )
        {
            EX_Log( "SMS_SMSView::OnDismissConfirmationScreen( ) : " + e.message );
        }
    },

    /******************************************************************************
      SendSMS

        Handler for the "Send" button. Sends an SMS request off to the server.

    ******************************************************************************/
    handleSendSMS : function( )
    {
        if ( document.getElementById( 'internationalRadio' ).checked )
        {
            this.SendInternationalMessage( );
        }
        else
        {
            this.SendDomesticMessage( );
        }

        var contactPopup = document.getElementById( 'contactPopup' );
        VM_SetPreference( PREF_KEY_LAST_CONTACT_KEY, contactPopup.options[ contactPopup.selectedIndex ].value );

        document.getElementById( "doButton" ).disabled = false;
    },

    /******************************************************************************
      ValidateInternationalSend

        Validates the configuration for an international message.

    ******************************************************************************/
    ValidateInternationalSend : function( )
    {
        var phoneNumberObj = document.getElementById( "recipientPhoneNumber" );

        var internationalNumber = UTILS_StripNonDigits( phoneNumberObj.value );

        // validate the number: >= 8 Characters -> valid.
        if ( ! internationalNumber || ( internationalNumber.length < 8 ) )
        {
            this.SetErrorMessage( "Please enter a valid international phone number." );

            this.m_confirmationScreenDismissalAction = function( )
            {
                phoneNumberObj.focus( );
                phoneNumberObj.select( );
            };

            return false;
        }

        return this.ValidateMessage( );
    },

    /******************************************************************************
      SendInternationalMessage

        Sends an international SMS message.

    ******************************************************************************/
    SendInternationalMessage : function( )
    {
        if ( ! this.ValidateInternationalSend( ) )
        {
            return;
        }

        var recipientPhoneNumber = document.getElementById( "recipientPhoneNumber" ).value;
        var message = document.getElementById( 'messageText' ).value;

        this.SendSMS( recipientPhoneNumber, INTERNATIONAL_CARRIER_ID, message );
    },

    /******************************************************************************
      ValidateDomesticMessage

        Validates the configuration for a domestic message.

    ******************************************************************************/
    ValidateDomesticMessage : function( )
    {
        var phoneNumberObj = document.getElementById( 'recipientPhoneNumber' );

        var fullNumber = UTILS_StripNonDigits( phoneNumberObj.value );

        if ( ! fullNumber || ( fullNumber.length == 0 ) )
        {
            this.SetErrorMessage( "Please enter a valid phone number." );

            this.m_confirmationScreenDismissalAction = function( )
            {
                phoneNumberObj.focus( );
                phoneNumberObj.select( );
            };

            return false;
        }

        if ( fullNumber.length != 10 && ( fullNumber.length != 11 || fullNumber.charAt( 0 ) !== '1' ) )
        {
            this.SetErrorMessage( "Please enter a valid area code." );

            this.m_confirmationScreenDismissalAction = function( )
            {
                phoneNumberObj.focus( );
                phoneNumberObj.select( );
            };

            return false;
        }

        var carrierId = SMS_UTILS_GetSelectedValueFromPopup( document.getElementById( 'carrierPopup' ) );

        if ( this.m_showCarriers && carrierId == INVALID_CARRIER_ID )
        {
            this.SetErrorMessage( "Please select a carrier." );
            return false;
        }

        return this.ValidateMessage( );
    },

    /******************************************************************************
      SendDomesticMessage

        Sends an domestic SMS message.

    ******************************************************************************/
    SendDomesticMessage : function( )
    {
        if ( ! this.ValidateDomesticMessage( ) )
        {
            return;
        }

        var recipientPhoneNumber = document.getElementById( "recipientPhoneNumber" ).value;

        var carrierId = OTHER_CARRIER_ID;

        if ( this.m_showCarriers )
        {
            carrierId = SMS_UTILS_GetSelectedValueFromPopup( document.getElementById( 'carrierPopup' ) );
        }

        var message = document.getElementById( 'messageText' ).value;

        this.SendSMS( recipientPhoneNumber, carrierId, message );

        if ( this.m_showCarriers )
        {
            VM_SetPreference( recipientPhoneNumber, SMS_UTILS_GetSelectedValueFromPopup( carrierPopup ) );
        }
    },

    /******************************************************************************
      ValidateMessage

        Validates the message in preparation for sending

    ******************************************************************************/
    ValidateMessage : function( )
    {
        var message = document.getElementById( 'messageText' ).value;

        if ( ! message || ( message.length == 0 ) )
        {
            this.SetErrorMessage( "Please enter a message!" );

            return false;
        }

        return true;
    }
};
